from dataclasses import dataclass
import inspect
import os
from typing import List


@dataclass
class DatabuttonSchedule:
    # Unique identifier
    uid: str
    # The name of the function
    name: str
    # Seconds between runs (first run will be at t0 + seconds)
    seconds: float
    # The path to the .py file where the schedule is defined
    filepath: str
    # The name of the module (i.e jobs.schedule)
    module_name: str

    def to_dict(self):
        return self.to_dict()


_schedules: List[DatabuttonSchedule] = []


def repeat_every(seconds: float):
    def wrapper(func):
        filepath = os.path.relpath(inspect.getfile(func), os.curdir)
        uid = f"{filepath}-{func.__name__}"
        _schedules.append(
            DatabuttonSchedule(
                uid=uid,
                name=func.__name__,
                seconds=seconds,
                filepath=filepath,
                module_name=inspect.getmodule(func).__name__,
            )
        )
        return func

    return wrapper
