from fastapi import FastAPI, Response, status
from fastapi.responses import RedirectResponse
from fastapi.staticfiles import StaticFiles
from fastapi.middleware.cors import CORSMiddleware

from databutton.server.processes import StreamlitProcess
from databutton.utils.build import (
    load_data_routers_for_app,
    load_schedules,
    read_artifacts_json,
)
from databutton_web import get_static_file_path

app = FastAPI()

app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

components = read_artifacts_json()

load_data_routers_for_app(app, components)
load_schedules(components)
# Set up proxies for streamlit apps
# They are ran as containers somewhere
for st in components.streamlit_apps:
    p = StreamlitProcess(st.route, "", st.port)
    p.add_route_to_app(app)


@app.get("/")
async def index():
    return RedirectResponse(url="/index.html")


app_dir = get_static_file_path()
app.mount("/static", StaticFiles(directory=".databutton"), name=".databutton")
app.mount("/", StaticFiles(directory=app_dir), name="app")


@app.get("/healthz")
async def healthz():
    return Response(status_code=status.HTTP_200_OK)
