from typing import TYPE_CHECKING

from databutton.decorators.streamlit import StreamlitApp

if TYPE_CHECKING:
    from databutton.utils.build import ArtifactDict

kube_request_resources = {
    "requests": {"memory": "512Mi", "cpu": "0.5"},
    "limits": {"memory": "512Mi", "cpu": "0.5"},
}


def generate_manifest(artifacts: "ArtifactDict"):
    streamlit_apps = []
    for app in artifacts.streamlit_apps:
        st_app = {
            "name": f"route-{app.uid}",
            "image": "IMAGE_NAME",
            "args": [
                app.filename,
                "--server.port",
                f"{app.port}",
                "--server.address",
                "0.0.0.0",
            ],
            "ports": [{"containerPort": app.port}],
            "env": [{"name": "PYTHONPATH", "value": "."}],
            "command": ["streamlit", "run"],
        }
        st_app["resources"] = {
            "requests": {
                "memory": app.memory,
                "cpu": app.cpu,
            },
            "limits": {"memory": app.memory, "cpu": app.cpu},
        }
        streamlit_apps.append(st_app)

    deployment = {
        "apiVersion": "apps/v1",
        "kind": "Deployment",
        "metadata": {"name": "PROJECT_ID", "labels": {"app": "PROJECT_ID"}},
        "spec": {
            "replicas": 1,
            "selector": {"matchLabels": {"app": "PROJECT_ID"}},
            "template": {
                "metadata": {
                    "labels": {"project_id": "PROJECT_ID", "app": "PROJECT_ID"}
                },
                "spec": {
                    "containers": streamlit_apps
                    + [
                        {
                            "name": "main",
                            "image": "IMAGE_NAME",
                            "ports": [{"containerPort": 8000}],
                            "command": ["databutton", "serve"],
                            "env": [{"name": "PYTHONPATH", "value": "."}],
                            "resources": kube_request_resources,
                        }
                    ]
                },
            },
        },
    }

    mapping = {
        "apiVersion": "getambassador.io/v3alpha1",
        "kind": "Mapping",
        "metadata": {
            "name": "PROJECT_ID",
        },
        "spec": {
            "hostname": "PROJECT_ID.dbtn.app",
            "service": "PROJECT_ID",
            "prefix": "/",
            "allow_upgrade": ["websocket"],
        },
    }

    service = {
        "apiVersion": "v1",
        "kind": "Service",
        "metadata": {
            "name": "PROJECT_ID",
        },
        "spec": {
            "ports": [{"port": 80, "targetPort": 8000}],
            "selector": {"app": "PROJECT_ID"},
        },
    }

    return {"deployment": deployment, "service": service, "mapping": mapping}
